

#include <MozziGuts.h>
#include <Oscil.h> // oscillator 
#include <tables/cos2048_int8.h> // table for Oscils to play
#include <Smooth.h>
#include <AutoMap.h> // maps unpredictable inputs to a range

//sinewave tables
#include <tables/sin2048_int8.h> // sine table for oscillator
// use: Oscil <table_size, update_rate> oscilName (wavetable), look in .h file of table #included above
Oscil <SIN2048_NUM_CELLS, AUDIO_RATE> aSin(SIN2048_DATA);//first sine wave - aSin
Oscil <SIN2048_NUM_CELLS, AUDIO_RATE> aSin2(SIN2048_DATA);//second sinewave aSin2

// use #define for CONTROL_RATE, not a constant
#define CONTROL_RATE 128 // powers of 2 please
float centre_freq = 440.0; //for the sine modulator

// desired carrier frequency max and min, for AutoMap
const int MIN_CARRIER_FREQ = 10;
const int MAX_CARRIER_FREQ = 1200;

// desired intensity max and min, for AutoMap, note they're inverted for reverse dynamics
const int MIN_INTENSITY = 0;
const int MAX_INTENSITY2 = 1000;

// desired mod speed max and min, for AutoMap, note they're inverted for reverse dynamics
const int MIN_MOD_SPEED = 20000;
const int MAX_MOD_SPEED = 5;

AutoMap kMapCarrierFreq(0,1023,MAX_CARRIER_FREQ, MIN_CARRIER_FREQ);
AutoMap kMapIntensity(0,1023,MAX_INTENSITY2,MIN_INTENSITY);
AutoMap kMapModSpeed(0,1023,MIN_MOD_SPEED,MAX_MOD_SPEED);


///pins and things
const int ledPin =  2;// the number of the LED pin
int tempoPin = A1;
const int Pitch_Pin = A3; //PITCH set the input for the knob
const int intensity_pin = A2; // INTENSITYset the analog input for fm_intensity
const int LFO_Pin = A0; // RATE set the analog input for mod rate
int mode1Pin = A4;//two selector switches
int mode2Pin = A5;
int ModeSelector1 = analogRead(mode1Pin);
int ModeSelector2 = analogRead(mode2Pin);
int ModeSel_01 = 0;

int modePin = 3; 
int modeSwitch = LOW; //int testD10 = digitalRead(10);
int volumeCont = 1; //volume control
int triggerPin = 4;
// Variables will change :
int ledState = LOW;             // ledState used to set the LED
int pulseOn = 0;
int pulseWidth = 0;
// Generally, you should use "unsigned long" for variables that hold time
// The value will quickly become too large for an int to store
unsigned long previousMillis = millis();  
unsigned long previousMillis2  = millis(); // will store last time LED was updated

// constants won't change :
unsigned long interval = 1000;


Oscil<COS2048_NUM_CELLS, AUDIO_RATE> aCarrier(COS2048_DATA);
Oscil<COS2048_NUM_CELLS, AUDIO_RATE> aModulator(COS2048_DATA);
Oscil<COS2048_NUM_CELLS, CONTROL_RATE> kIntensityMod(COS2048_DATA);

int mod_ratio = 5; // brightness (harmonics)
long fm_intensity; // carries control info from updateControl to updateAudio

// smoothing for intensity to remove clicks on transitions
float smoothness = 0.95f;
Smooth <long> aSmoothIntensity(smoothness);


void setup(){
//Serial.begin(9600); // for Teensy 3.1, beware printout can cause glitches
 // Serial.begin(115200); // set up the Serial output so we can look at the piezo values // set up the Serial output so we can look at the light level
  startMozzi(); // :))
  pinMode(triggerPin, OUTPUT);
}


void updateControl(){

ModeSelector1 = mozziAnalogRead(mode1Pin);
ModeSelector1 = map(ModeSelector1, 0, 1200, 0, 8);
ModeSel_01 = ModeSelector1;

ModeSelector2 = mozziAnalogRead(mode2Pin);

interval = mozziAnalogRead(tempoPin);
  //volumeCont = 1;
  // read the knob
  int Pitch_value = mozziAnalogRead(Pitch_Pin); // value is 0-1023
  //inverted for the sinewave
    int Pitch_value2 = map(Pitch_value, 0, 1200, 2000, 2);
   

   
    
  // map the knob to carrier frequency
  int carrier_freq = kMapCarrierFreq(Pitch_value);
  int mod_freq = 0;
  
//if select 4 octave the osc fm sound
 if(ModeSel_01 == 1){
    carrier_freq = carrier_freq * 2;
 }
  //previous settings for mod
mod_freq = carrier_freq * mod_ratio;
 // }

  
  // set the FM oscillator frequencies
  //VOLUME CONTROL / DIGITAL INPUT X FREQ BY EITHER 1 OR 0 TO MUTE THE SOUND
    aCarrier.setFreq((carrier_freq)* volumeCont);
  //aCarrier.setFreq(carrier_freq);
   
  aModulator.setFreq(mod_freq);
   

   
  // read the light dependent resistor on the width Analog input pin
  int intensity_value= mozziAnalogRead(intensity_pin); // value is 0-1023
  

  int intensity_calibrated = kMapIntensity(intensity_value);
  
  
 // calculate the fm_intensity
  fm_intensity = ((long)intensity_calibrated * (kIntensityMod.next()+128))>>8; // shift back to range after 8 bit multiply
 
  
  // read the light dependent resistor on the speed Analog input pin
  int Modulation_value = mozziAnalogRead(LFO_Pin); // value is 0-1023


 
  // use a float here for low frequencies
float mod_speed = (float)kMapModSpeed(Modulation_value)/1000;
//add this to the sinewave modualation
   //set the frequency for the sine wave option 1
 if(ModeSel_01 == 4){ 
  pulseWidth = intensity_calibrated;    
    aSin.setFreq(Pitch_value2 - Modulation_value + intensity_calibrated); // set the frequency low for the sine
    }
 
 
 //add vibrato to the sinewave if selected
 if(ModeSel_01 == 2){
float depth = (map(intensity_calibrated,MIN_INTENSITY,MAX_INTENSITY2, 0, 20) );
aSin2.setFreq((mod_speed * 200)); // set the frequency
float vibrato = depth * aSin2.next();
aSin.setFreq(Pitch_value2 + vibrato); // set the frequency
  }//END SELECT 2
 
  //SELLECT 3 OCTAVE HIGHER
  if(ModeSel_01 == 3){
float depth = (map(intensity_calibrated,MIN_INTENSITY,MAX_INTENSITY2, 0, 20) );
aSin2.setFreq((mod_speed * 200)); // set the frequency
float vibrato = depth * aSin2.next();
aSin.setFreq((Pitch_value2 * 2) + vibrato); // set the frequency
  }
  
  
  
  kIntensityMod.setFreq(mod_speed);
  }


int updateAudio(){
long modulation = (aSmoothIntensity.next(fm_intensity) * aModulator.next()) * volumeCont;

//mode switch to select sound type played
 switch (ModeSel_01) {
      case 0:
       if(pulseOn == 1){
    return aCarrier.phMod(modulation);
       }
        break;
     case 4:
     //PLAY THE SINE WAVE IF THE PULSE IS ON
     if(pulseOn == 1){
     return aSin.next(); // return an int signal centred around 0
     }
     break;
      case 2:
     //PLAY THE SINE WAVE WITH MODULATION IF THE PULSE IS ON
     if(pulseOn == 1){
     return aSin.next(); // return an int signal centred around 0
     }
       break;
       case 3:
     //PLAY THE SINE WAVE WITH MODULATION IF THE PULSE IS ON
     if(pulseOn == 1){
     return aSin.next(); // return an int signal centred around 0
     }
      break;
      case 1:
       if(pulseOn == 1){
    return aCarrier.phMod(modulation);
       }
        break;
   
 }


}


void loop(){
  int pulseRatio = 30;
 
   audioHook();
 switch (ModeSel_01) {
      case 0:
    
        break;
     case 1:
        break;
      //default:
 }

  
  
 modeSwitch = digitalRead(modePin);
   unsigned long currentMillis = millis();


if(modeSwitch == HIGH){
  
if (currentMillis - previousMillis >= interval) {
    // save the last time you blinked the LED
    //previousMillis2 = previousMillis;
    previousMillis = currentMillis;
    // if the LED is off turn it on and vice-versa:
    if (ledState == LOW) {
      ledState = HIGH;
      //volumeCont = 1;
      pulseOn = 1;
      digitalWrite(triggerPin, HIGH);

      
    } else {
      ledState = LOW;
      //volumeCont = 0;
      pulseOn = 0;
       digitalWrite(triggerPin, LOW);
    }

//if mode 1 selected use the intensity control to change the pulse width by turning it off
    //if(ModeSel_01 == 1){
    //disabled mode below
      if(ModeSel_01 == 23){
      if (currentMillis - previousMillis2 >= pulseWidth) {
      ledState = LOW;
      //volumeCont = 0;
      pulseOn = 0;
       digitalWrite(triggerPin, LOW);
         previousMillis2 = currentMillis;
    }}//END IF MODE SELECTED AT 1 FUNCTION

    // set the LED with the ledState of the variable:
    digitalWrite(ledPin, ledState);
  }
}}




